#!/usr/bin/python
import subprocess
import os
import os.path
import re
import math
import grp
import pwd
import time
from datetime import date
from datetime import datetime
import sys, getopt


ORABASE = "/u01/app/oracle"
ORAHOME = "/u01/app/oracle/product/12.2.0/dbhome_1"
RELEASE = subprocess.check_output("cat /etc/system-release", shell=True)
KERNEL = subprocess.check_output("uname -r", shell=True)
PROCESSOR = subprocess.check_output("/bin/cat /proc/cpuinfo | grep -i vendor_id", shell=True)
GRUB1_DIR = "/boot/grub"
GRUB2_DIR = "/boot/grub2"
GRUB1_FILE = "/boot/grub/grub.conf"
GRUB2_FILE = "/boot/grub2/grub.cfg"
GRUB2_OPTIONS = "/etc/default/grub"
USERID="54321"
GROUPID="54321"
RESULT=0

ORACLE_VERSION=""
ORACLE_SUPPORTED_VERSION=["12cR2"]

# Printing colors
# ex. 
#print (colors.GREEN + "[PASSED]" + colors.END)
class colors:
		GREEN = '\033[92m'
		RED = '\033[91m'
		END = '\033[0m'
		NAME = '\033[94m'

		def disable(self):
				self.GREEN = ''
				self.RED = ''
				self.END = ''

def f_executecommand(command): 
		executecommand = subprocess.Popen(command,stdout=subprocess.PIPE,shell=True)
		output, error = executecommand.communicate()
		if executecommand.returncode != 0:
				f = open('preinstall_log.txt','w')
				print "execution of command %s failed" % (command)
				f.write("execution of command %s failed" % (command))
				f.close()
				sys.exit(2)
		else:
				return output

def f_backup(filepath):

                f = open('log.txt','a+')
                txt = colors.NAME + "\nBacking up Files" + colors.END
                f.write(txt)
                print (txt)
                timenow = datetime.now().strftime("%y-%m-%d_%H:%M")
                command = "cp " + filepath + " " + filepath + "_" + str(timenow)
                f_executecommand(command)

                print ("Back up files" + colors.GREEN + "\t\t\t\t\t[COMPLETED]" + colors.END)
                f.write("Back up files" + colors.GREEN + "\t\t\t\t\t[COMPLETED]" + colors.END)
                f.close()

def f_checkrelease():

		f = open('log.txt','a+')
		txt = colors.NAME + "\nChecking OS Release Version" + colors.END
		f.write(txt)
		print (txt)
		if os.path.isfile("/etc/oracle-release"):
				distribution = "Oracle Linux"
				value = re.findall(r'\d+', RELEASE)
				version = '.'.join(value)
                                if version < '7':
                                    print ("OS Version %s not compatible with RPM") % (version)
                                    sys.exit(2)
                                if "uek" in KERNEL.split():
				    print ("OS = %s, Version = %s" + colors.RED + "\t\t\t[FAILED]" + colors.END) % (distribution, version)
				    print "RPM not Compatible with UEK Kernel. Aborting execution"
				    f.write(("OS = %s, Version = %s" + colors.RED + "\t\t\t[FAILED]" + colors.END) % (distribution, version))
				    f.write("RPM Not Compatible with UEK Kernel. Aborting execution")
				    sys.exit(2)
                                return version

		if os.path.isfile("/etc/redhat-release"):
				distribution = "Red Hat"
				value = re.findall(r'\d+', RELEASE)
				version = '.'.join(value)
                                if version < '7':
                                     print ("OS Version %s not compatible with RPM") % (version)
                                     sys.exit(2)
				print ("OS = %s, Version = %s" + colors.GREEN + "\t\t\t[PASSED]" + colors.END) % (distribution, version)
				f.write(("OS = %s, Version = %s" + colors.GREEN + "\t\t\t[PASSED]" + colors.END) % (distribution, version))
				return version

		f.close()

def f_checkgrub():

		f = open('log.txt','a+')
		txt = colors.NAME + "\nChecking Grub" + colors.END
		f.write(txt)
		print (txt) 
		if os.path.isfile(GRUB2_FILE):
				version = 2
				grub_dir = GRUB2_DIR
				grub_file = GRUB2_FILE
				print ("Grub version = 2" + colors.GREEN + "\t\t\t\t[PASSED]" + colors.END)
				f.write("Grub version = 2" + colors.GREEN + "\t\t\t\t[PASSED]" + colors.END)
                                f_backup(GRUB2_OPTIONS)


		elif os.path.isfile(GRUB1_FILE):
				version = 1
				grub_dir = GRUB1_DIR
				grub_file = GRUB1_FILE
				print ("Grub version = 1" + colors.RED + "\t\t\t\t[FAILED]" + colors.END)
				f.write("Grub version = 1" + colors.RED + "\t\t\t\t[FAILED]" + colors.END)
                                sys.exit(2)
	
		else:
				print "Cannot find grub version. Aborting"
				print ("Cannot find grub version. Aborting execution" + colors.RED + "\t\t\t[FAILED]" + colors.END)
				f.write("Cannot find grub version. Aborting execution" + colors.RED + "\t\t\t[FAILED]" + colors.END)
				sys.exit(2)

		f.close()

def f_modifygrub():

                s = "transparent_hugepage=never numa=off"

      		input = file(GRUB2_OPTIONS,'r')
		saved_input = []
		for eachline in input:
		    saved_input.append(eachline)
                input.close()

		lenfile = len(saved_input)

		for i in range(0, lenfile):
		  if 'GRUB_CMDLINE_LINUX' in saved_input[i]:
		     newline = saved_input[i].split('"')[0] + '"' + saved_input[i].split('"')[1] + " " + s + '"'
		     replace = re.sub(saved_input[i],newline,saved_input[i])
		     if replace is not None:
		        saved_input[i] = replace + "\n"


		input = file(GRUB2_OPTIONS,'w')
		for each in saved_input:
		   input.write(each)

                input.close()
                print ("Modified Grub File Options" + colors.GREEN + "\t\t\t[PASSED]" + colors.END)

def f_grub2mkconfig():
   
                command = "/usr/sbin/grub2-mkconfig -o " + GRUB2_FILE
                f_executecommand(command)
                print ("Reconfigured GRUB CFG" + colors.GREEN + "\t\t\t\t[PASSED]" + colors.END)

def f_nozeroconf():
                f = open('/etc/sysconfig/network','a+')
                f.write('NOZEROCONF=yes\n')
                f.close()

def f_checkhwresrouces():

		f = open('log.txt','a+')
		txt = (colors.NAME + "\nChecking Hardware Resources" + colors.END)
		f.write(txt)
		print (txt)
		ram = subprocess.check_output("grep MemTotal /proc/meminfo  | awk -F ':' '{print $2}'", shell=True)
		swap = subprocess.check_output("grep SwapTotal /proc/meminfo | awk -F ':' '{print $2}'", shell=True)
		ramsize = re.findall(r'\d+', ram)
		swapsize = re.findall(r'\d+', swap)
		arch = subprocess.check_output("uname -m", shell=True)
	 
		if ramsize < 2097152:
				if swapsize < (1.5 * ramsize):
						print (colors.RED + "swapsize check \t\t\t[FAILED]" + colors.END )
						sys.exit(2)

		if ramsize > 2097152 and ramsize < 16777216:
				if swapsize < ramsize:
						print (colors.RED + "swapsize check \t\t\t[FAILED]" + colors.END )
						print "There is not enough swapsize. swapsize should atleast be same as ramsize. Aborting"
						f.write(colors.RED + "swapsize check \t\t\t[FAILED]" + colors.END )
						f.write( "There is not enough swapsize. swapsize should atleast be same as ramsize. Aborting")
						sys.exit(2)

		if ramsize > 16777216:
				if swapsize < 16777216:
						print (colors.RED + "swapsize check \t\t\t[FAILED]" + colors.END )
						print "There is not enough swapsize. Swapsize should atleast be 16G. Aborting"
						f.write(colors.RED + "swapsize check \t\t\t[FAILED]" + colors.END )
						f.write("There is not enough swapsize. Swapsize should atleast be 16G. Aborting")
						sys.exit(2)
						

		
		freespace = subprocess.check_output("df -h /tmp | awk -F ' ' '{print $4}' | tail -1", shell=True)
		size = re.findall(r'\d+', freespace)
		if size < 1:
				print "/tmp check - FAILED"
				f.write("/tmp check - FAILED")
				sys.exit(2)
		print ("Free memory, Swap size, /tmp" + colors.GREEN +  "\t\t\t[PASSED]" + colors.END)
		f.write("Free memory, Swap size, /tmp" + colors.GREEN +  "\t\t\t[PASSED]" + colors.END)
		f.close

def f_checkjava():
		f = open('log.txt','a+')
		txt = colors.NAME + "\nChecking Java Version" + colors.END
		f.write(txt)
		print (txt)
		java = subprocess.check_output("java -version > /tmp/java 2>&1", shell=True)
		version = subprocess.check_output("head -n  1 /tmp/java  | awk -F '_' '{print $1}'", shell=True)
		javaversion = re.findall(r'\d+', version)
		javavers = '.'.join(javaversion)
		if javavers < "1.6.0":
			 print "Current Java Version is %s. Update to 1.6.0 or higher. Aborting " % (javavers) + colors.RED + "[FAILED]" + colors.END
			 f.write("Current Java Version is %s. Update to 1.6.0 or higher. Aborting " % (javavers) + colors.RED + "[FAILED]" + colors.END)
			 exit(0)
		print ("Java version - OK" + colors.GREEN + "\t\t\t\t[PASSED]" + colors.END)
		f.write("Java version - OK" + colors.GREEN + "\t\t\t\t[PASSED]" + colors.END)
		f.close()
	


def f_osgroupsusers():
		f = open('log.txt','a+')
		txt = colors.NAME + "\nAdding OS Users" + colors.END
		f.write(txt)
		print(txt)
		groupsinfo = {'oinstall':'54321','oper':'54326','dba':'54322'}
		for group in groupsinfo.keys():
			 try: 
					grp.getgrnam(group)
			 except KeyError:
					command = "groupadd -g " + groupsinfo[group] + " " + group
					f_executecommand(command)
		
		usersinfo = {'oracle':'54321'}
		for user in usersinfo.keys():
			 try:
					pwd.getpwnam(user)
			 except KeyError:
					command = "useradd -u " + usersinfo[user] + " -g oinstall " + user
					f_executecommand(command)

		usersgroups = {'oracle':['oinstall','oper', 'dba']}
		for user in usersinfo.keys():
			 command = "usermod -G " + ','.join(usersgroups[user]) + " " + user
			 f_executecommand(command)

		print ("Added users" + colors.GREEN + "\t\t\t\t\t[COMPLETED]" + colors.END)
		f.write("Added users" + colors.GREEN + "\t\t\t\t\t[COMPLETED]" + colors.END) 
		f.close 

def f_d_osgroupsusers():

		f = open('log.txt','a+')
		txt = colors.NAME + "\Deleting OS Users" + colors.END
		f.write(txt)
		usersinfo = ["oracle"]
		for user in usersinfo:
			 try:
					pwd.getpwnam(user)
					command = "userdel " + user
					f_executecommand(command)
			 except KeyError:
					continue

		groupsinfo = ['oinstall','oper','dba']
		for group in groupsinfo:
			 try:
					grp.getgrnam(group)
					command = "groupdel " + group
					f_executecommand(command)
			 except KeyError:
					continue

		print ("Deleted users" + colors.GREEN + "\t\t\t\t\t[COMPLETED]" + colors.END)
		f.write("Deleted users" + colors.GREEN + "\t\t\t\t\t[COMPLETED]" + colors.END) 
		f.close 


		
def f_requireddirectories():

		f = open('log.txt','a+')
		txt = (colors.NAME + "\nCreating Required Directories" + colors.END)
		f.write(txt)
		print (txt)

		uid = pwd.getpwnam("oracle").pw_uid
		gid = grp.getgrnam("oinstall").gr_gid
		dirs = [ORABASE, ORAHOME]
		for eachdir in dirs:
				try:
					 os.stat(eachdir)
				except:
					 os.makedirs(eachdir) 
		for eachdir in dirs:
                    command = "chown -R oracle:oinstall " + eachdir
                    f_executecommand(command)

		parentdir=os.path.dirname(os.path.normpath(ORABASE))
		os.chown(parentdir, uid, gid)

		print( "Required Directories" + colors.GREEN + "\t\t\t\t[COMPLETED]" + colors.END) 
		f.write( "Required Directories" + colors.GREEN + "\t\t\t\t[COMPLETED]" + colors.END)  
		f.close()   

def f_kernelparams():

                f = open('/etc/sysctl.conf','a+')
                f.write('''# oracle-rdbms-server-12cR2-preinstall setting for fs.file-max is 6815744
fs.file-max = 6815744
#
# # oracle-rdbms-server-12cR2-preinstall setting for kernel.sem is '250 32000 100 128'
kernel.sem = 250 32000 100 128
#
# # oracle-rdbms-server-12cR2-preinstall setting for kernel.shmmni is 4096
kernel.shmmni = 4096
#
# # oracle-rdbms-server-12cR2-preinstall setting for kernel.shmall is 1073741824 on x86_64
kernel.shmall = 1073741824
#
# # oracle-rdbms-server-12cR2-preinstall setting for kernel.shmmax is 4398046511104 on x86_64
kernel.shmmax = 4398046511104
#
# # oracle-rdbms-server-12cR2-preinstall setting for kernel.panic_on_oops is 1 per Orabug 19212317
kernel.panic_on_oops = 1
#
# # oracle-rdbms-server-12cR2-preinstall setting for net.core.rmem_default is 262144
net.core.rmem_default = 262144
#
# # oracle-rdbms-server-12cR2-preinstall setting for net.core.rmem_max is 4194304
net.core.rmem_max = 4194304
#
# # oracle-rdbms-server-12cR2-preinstall setting for net.core.wmem_default is 262144
net.core.wmem_default = 262144
#
# # oracle-rdbms-server-12cR2-preinstall setting for net.core.wmem_max is 1048576
net.core.wmem_max = 1048576
#
# # oracle-rdbms-server-12cR2-preinstall setting for net.ipv4.conf.all.rp_filter is 2
net.ipv4.conf.all.rp_filter = 2
#
# # oracle-rdbms-server-12cR2-preinstall setting for net.ipv4.conf.default.rp_filter is 2
net.ipv4.conf.default.rp_filter = 2
#
# # oracle-rdbms-server-12cR2-preinstall setting for fs.aio-max-nr is 1048576
fs.aio-max-nr = 1048576
#
# # oracle-rdbms-server-12cR2-preinstall setting for net.ipv4.ip_local_port_range is 9000 65500
net.ipv4.ip_local_port_range = 9000 65500 \n''')
                f.close()

def f_d_requireddirectories():

		ORABASE = "/u01/app/oracle"
		command = "rm -rf " + ORABASE
		f_executecommand(command)

def f_executeall():
		

		f_checkrelease()
                f_checkhwresrouces()
                f_checkjava()
                f_checkgrub()
                f_modifygrub()
                f_grub2mkconfig()
                f_nozeroconf()
                f_kernelparams()
                f_osgroupsusers()
                f_requireddirectories()

def f_deleteall():
		f_d_osgroupsusers()

def main():
		
		 if len(sys.argv) == 1:
			 #print "execute the script with -a to create or -d to delete"
			 print "Usage: dell-redhat-rdbms-12cR2-preinstall [OPTION]"
			 print "\t-a\t\t\tInstall Oracle Prerequisites"
			 print "\t-d\t\t\tUninstall Oracle Prerequisites"
			 print "\t-r\t\t\tCheck Oracle Release"





		 if len(sys.argv) > 1:
			 myopts, args = getopt.getopt(sys.argv[1:],"adr:")

			 for option, value in myopts:
				if option == '-a':
				   f_executeall()


				if option == '-d':
				   f_deleteall()


				if option =='-r':
					print 'RELEASE'
					ORACLE_VERSION = sys.argv[2]
					print ORACLE_VERSION
					for i in ORACLE_SUPPORTED_VERSION:
						if i == ORACLE_VERSION:
							print "Oracle Version SUPPORTED!"
						else:
							print "Oracle Version NOT SUPPORTED!"
						
							 
					 
																						
if __name__ == '__main__':
	 main()

							
	 
